﻿using System;

namespace ClassUnderTest
{

    /// <summary>
    /// The delegate for method that handles receiving of a message
    /// </summary>
    /// <param name="sender">sender</param>
    /// <param name="message">new message</param>
    public delegate void MessageReceivedHandler(object sender, string message);


    /// <summary>
    /// The interface for communication channel
    /// </summary>
    public interface ICommunicationChannel
    {
        void Connect(string conStr);
        void Send(byte[] messageId, string message);
        event MessageReceivedHandler OnMessageReceived;
    }


    /// <summary>
    /// The class under test
    /// </summary>
    public class CommunicationAdapter 
    {

        // fields
        protected ICommunicationChannel channel;                    // communication channel  
        

        /// <summary>
        /// The event that is invoked when a message is received from channel
        /// </summary>
        public event MessageReceivedHandler OnMessageReceived;


        /// <summary>
        /// The constructor
        /// </summary>
        public CommunicationAdapter()
        {
            channel = new CommunicationChannel();            
        }


        /// <summary>
        /// The property that determines whether connection string is valid
        /// </summary>
        public bool IsConnStrValid { get; private set; }

        /// <summary>
        /// The property with connection string
        /// </summary>
        public string ConnectionString { get; private set; }


        /// <summary>
        /// The method that validates connection string
        /// </summary>
        /// <param name="conStr">connection string</param>
        /// <returns>true whether it is valid (contains '@' character), false otherwise</returns>
        public bool SetAndValidateConStr(string conStr)
        {
            if (conStr == null) throw new ArgumentNullException();
            if (!conStr.Contains("@")) return false;
            IsConnStrValid = true;
            ConnectionString = conStr;
            return true;
        }


        /// <summary>
        /// The method that establishes connection
        /// </summary>           
        public void Connect()
        {
            if (!IsConnStrValid) throw new InvalidOperationException("Connection string is not valid");
            channel.OnMessageReceived += MessageReceivedHandler;
            channel.Connect(ConnectionString);            
        }


        /// <summary>
        /// The method that sends a message to the channel
        /// </summary>
        /// <param name="message">message</param>
        public bool Send(string message)
        {
            try
            {
                byte[] msgId = new byte[3];
                Random random = new Random();
                msgId[0] = (byte) random.Next(255);
                msgId[1] = (byte) random.Next(255);
                msgId[2] = (byte) (msgId[0] ^ msgId[1]);
                channel.Send(msgId, message);
                return true;
            }
            catch (Exception)
            {
                return false;
            }                    
        }


        /// <summary>
        /// The method that handles receiving of a message from channel
        /// </summary>
        /// <param name="sender">sender</param>
        /// <param name="message">received message</param>
        protected void MessageReceivedHandler(object sender, string message)
        {
            if (OnMessageReceived != null)
                OnMessageReceived(this, message);
        }

    }

}
