﻿using System;
using ClassUnderTest;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;
using Rhino.Mocks.Constraints;
using Rhino.Mocks.Interfaces;

namespace ClassUnderTestTests
{
    
    
    /// <summary>
    /// The class that contains tests of CommunicationAdapter class
    ///</summary>
    [TestClass]
    public class CommunicationAdapterTest
    {


        #region test initialization and test cleanup

        // tested class
        private TestableAdapter adapter;
        

        /// <summary>
        /// The method that initializes test
        /// </summary>
        [TestInitialize]
        public void Initialize()
        {
            adapter = new TestableAdapter();
        }
        
        /// <summary>
        /// The method that cleanup test
        /// </summary>
        [TestCleanup]
        public void Cleanup()
        {
            adapter = null;
        }        

        #endregion


        #region simple tests with class Assert

        /// <summary>
        /// Tests call of SetAndValidateConStr with correct constr (returns true)
        ///</summary>
        [TestMethod, TestCategory("Assert")]
        public void SetAndValidateConStr_CorrectConStr_ReturnTrue()
        {
            // arrange 
            string correctConStr = "Correct@string";

            // act
            bool result = adapter.SetAndValidateConStr(correctConStr);

            // assert
            Assert.IsTrue(result, "Verification of correct constr fails");
        }       


        /// <summary>
        /// Tests call of SetAndValidateConStr with incorect constr (returns false)
        ///</summary>
        [TestMethod, TestCategory("Assert")]
        public void SetAndValidateConStr_InvalidConStr_ReturnFalse()
        {
            // arrange 
            string incorrectConStr = "Incorrect_string";

            // act
            bool result = adapter.SetAndValidateConStr(incorrectConStr);

            // assert
            Assert.IsFalse(result, "Verification of incorrect constr passes");
        }

        /// <summary>
        /// Tests state of adapter after call of SetAndValidateConStr with correct constr (IsConnStrValid prop. is true)
        ///</summary>      
        [TestMethod, TestCategory("Assert")]
        public void SetAndValidateConStr_CorrectConStr_IsConnStrValidIsTrue()
        {
            // arrange 
            string correctConStr = "Correct@string";

            // act
            adapter.SetAndValidateConStr(correctConStr);

            // assert
            Assert.IsTrue(adapter.IsConnStrValid, "IsConnStrValid is not set to true");
        }


        /// <summary>
        /// Tests state of adapter after call of SetAndValidateConStr with correct constr (value is set to ConnectionString prop.)
        ///</summary>
        [TestMethod, TestCategory("Assert")]
        public void SetAndValidateConStr_CorrectConStr_ConStrIsSetToConnectionString()
        {
            // arrange 
            string correctConStr = "Correct@string";

            // act
            adapter.SetAndValidateConStr(correctConStr);

            // assert
            Assert.AreEqual(correctConStr, adapter.ConnectionString, "ConnectionString is not correcty set");
        }


        // todo: state of adapter after calling of SetAndValidateConStr with incorrect constr       

        #endregion


        #region testing of exceptions 

        /// <summary>
        /// Tests call of SetAndValidateConStr with null constr (throws ArgumentNullException)
        ///</summary>
        [TestMethod, ExpectedException(typeof(ArgumentNullException)), TestCategory("Exceptions")]
        public void SetAndValidateConStr_NullConStr_ThrowsArgumentNullException()
        {
            // act
            adapter.SetAndValidateConStr(null);
        }


        /// <summary>
        /// Tests call of Connect before constr is initialized
        ///</summary>
        [TestMethod, ExpectedException(typeof(InvalidOperationException)), TestCategory("Exceptions")]
        public void Connect_NonInitializedConStr_ThrowsInvalidOperationException()
        {
            // act
            adapter.Connect();
        }       

        #endregion


        #region testing with stabs

        /// <summary>
        /// Tests whether Send method returns true when no error occured during sending
        ///</summary>
        [TestMethod, TestCategory("Stub")]
        public void Send_SendingSuccessful_ReturnTrue()
        {            
            // arrange
            MockRepository mocks = new MockRepository();
            ICommunicationChannel channel = mocks.Stub<ICommunicationChannel>();
            adapter.Channel = channel;    
                   
            // act
            bool result = adapter.Send("message");    
            
            // assert
            Assert.IsTrue(result, "Successful sending has returned false");
        }


        /// <summary>
        /// Tests whether Send method returns false when error occured during sending
        ///</summary>
        [TestMethod, TestCategory("Stub")]
        public void Send_SendingFails_ReturnFalse()
        {
            // arrange
            MockRepository mocks = new MockRepository();
            ICommunicationChannel channel = mocks.Stub<ICommunicationChannel>();
            adapter.Channel = channel;

            // record
            using(mocks.Record())
            {
                channel.Send(null, "message");
                LastCall.IgnoreArguments();
                LastCall.Throw(new Exception("sending failed"));
            }

            // act
            bool result = adapter.Send("message");

            // assert
            Assert.IsFalse(result, "Failed sending has returned true");
        }   

        #endregion


        #region testing with mocks

        /// <summary>
        /// Tests whether Connect method correctly calls Connect method of channel
        ///</summary>
        [TestMethod, TestCategory("Mock")]
        public void Connect_ConStrIsSet_PassConStrToConnectMethodOfChannel()
        {
            // arrange
            MockRepository mocks = new MockRepository();
            ICommunicationChannel channel = mocks.DynamicMock<ICommunicationChannel>();
            adapter.Channel = channel;
            string correctString = "Correct@string";

            // record
            using (mocks.Record())
            {
                channel.Connect(correctString);                
            }

            // act
            adapter.SetAndValidateConStr(correctString);
            adapter.Connect();

            // replay
            mocks.Verify(channel);            
        }


        /// <summary>
        /// Tests correct generating of message ids
        ///</summary>
        [TestMethod, TestCategory("Mock")]
        public void Send_SendCall_CorrectCallWithValidMsgIdOnChannelSend()
        {
            // arrange
            MockRepository mocks = new MockRepository();
            ICommunicationChannel channel = mocks.DynamicMock<ICommunicationChannel>();
            adapter.Channel = channel;
            string message = "tested message";

            // record
            using (mocks.Record())
            {
                channel.Send(null, null);
                LastCall.Constraints(
                    Is.Matching<byte[]>(b => b.Length == 3 && (b[0] ^ b[1]) == b[2]),  
                    Is.Equal(message));
            }

            // act
            adapter.Send(message);           

            // replay
            mocks.Verify(channel);
        }   

        #endregion


        #region testing of events

        /// <summary>
        /// Tests that if channel invokes a message then adapter invokes the message
        ///</summary>
        [TestMethod, TestCategory("Event")]
        public void MessageReceivedHandler_ChannelInvokesEvent_AdapterInvokesEvent()
        {
            // arrange
            MockRepository mocks = new MockRepository();
            ICommunicationChannel channel = mocks.Stub<ICommunicationChannel>();
            adapter.Channel = channel;
            string expMessage = "tested message";

            // record
            IEventRaiser raiser;
            using (mocks.Record())
            {
                channel.OnMessageReceived += null;
                raiser = LastCall.GetEventRaiser();
            }

            // act
            bool fired = false;
            string actMessage = null;
            object sender = null;
            adapter.OnMessageReceived += ((s, m) =>
            {
                fired = true;
                actMessage = m;
                sender = s;
            });
            adapter.SetAndValidateConStr("correct@constr");
            adapter.Connect();
            raiser.Raise(channel, expMessage);

            // assert
            Assert.IsTrue(fired);
            Assert.AreEqual(expMessage, actMessage);
            Assert.AreSame(adapter, sender);
        }   

        #endregion

    }
}
